@@ -38,7 +38,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #include "tonezone.h"
 #else
 #include <zaptel/zaptel.h>
@@ -268,8 +268,8 @@ int dspanconfig(char *keyword, char *args)
 	}
 
 
-	strncpy(zds[numdynamic].driver, realargs[0], sizeof(zds[numdynamic].driver));
-	strncpy(zds[numdynamic].addr, realargs[1], sizeof(zds[numdynamic].addr));
+	zap_copy_string(zds[numdynamic].driver, realargs[0], sizeof(zds[numdynamic].driver));
+	zap_copy_string(zds[numdynamic].addr, realargs[1], sizeof(zds[numdynamic].addr));
 	zds[numdynamic].numchans = chans;
 	zds[numdynamic].timing = timing;
 	
@@ -377,7 +377,7 @@ int apply_channels(int chans[], char *argstr)
 	for (x=0;x<res;x++) {
 		if (strchr(args[x], '-')) {
 			/* It's a range */
-			strncpy(argcopy, args[x], sizeof(argcopy));
+			zap_copy_string(argcopy, args[x], sizeof(argcopy));
 			res2 = parseargs(argcopy, range, 2, '-');
 			if (res2 != 2) {
 				error("Syntax error in range '%s'.  Should be <val1>-<val2>.\n", args[x]);
@@ -571,7 +571,7 @@ static int chanconfig(char *keyword, char *args)
 				} else {
 					cc[x].sigtype = ZT_SIG_HDLCNET;
 					if (idle) {
-					    strncpy(cc[x].netdev_name, idle, sizeof(cc[x].netdev_name) - 1);
+					    zap_copy_string(cc[x].netdev_name, idle, sizeof(cc[x].netdev_name));
 					}
 					master = x;
 				}
@@ -631,7 +631,7 @@ static int registerzone(char *keyword, char *args)
 		error("Too many tone zones specified\n");
 		return 0;
 	}
-	strncpy(zonestoload[numzones++], args, sizeof(zonestoload[0]));
+	zap_copy_string(zonestoload[numzones++], args, sizeof(zonestoload[0]));
 	return 0;
 }
 
@@ -967,7 +967,7 @@ int rad_apply_channels(int chans[], char *argstr)
 	for (x=0;x<res;x++) {
 		if (strchr(args[x], '-')) {
 			/* It's a range */
-			strncpy(argcopy, args[x], sizeof(argcopy));
+			zap_copy_string(argcopy, args[x], sizeof(argcopy));
 			res2 = parseargs(argcopy, range, 2, '-');
 			if (res2 != 2) {
 				error("Syntax error in range '%s'.  Should be <val1>-<val2>.\n", args[x]);
@@ -1177,7 +1177,7 @@ static void printconfig(int fd)
 		for (x=1;x<ZT_MAX_CHANNELS;x++) 
 			if (cc[x].sigtype)
 				configs++;
-	printf("\n%d channels configured.\n\n", configs);
+	printf("\n%d channels to configure.\n\n", configs);
 }
 
 static struct handler {
@@ -1397,16 +1397,6 @@ int main(int argc, char *argv[])
 						current_state.channo = cc[x].chan | ZT_GET_PARAMS_RETURN_MASTER;
 						if (ioctl(fd, ZT_GET_PARAMS, &current_state))
 							needupdate = 1;
-						else if (current_state.chanpos == 16 &&
-							(cc[x].sigtype & ZT_SIG_CLEAR) && 
-							!(lc[current_state.spanno - 1].lineconfig &
-							(ZT_CONFIG_D4|ZT_CONFIG_ESF|ZT_CONFIG_B8ZS|ZT_CONFIG_CCS))) {
-								fprintf(stderr, 
-									"CAS signalling on span %u conflicts with %s on channel %u.\n",
-									current_state.spanno, 
-									sigtype_to_str(cc[x].sigtype), x);
-								exit(1);
-						}
 					}
 					
 					if (!needupdate) {
@@ -1461,7 +1451,8 @@ int main(int argc, char *argv[])
 						fflush(stdout);
 					}
 					if (tone_zone_register(fd, zonestoload[x]))
-						error("Unable to register tone zone '%s'\n", zonestoload[x]);
+						if (errno != EBUSY)
+							error("Unable to register tone zone '%s'\n", zonestoload[x]);
 				}
 				if (debug & DEBUG_APPLY) {
 					printf("Doing startup\n");
