@@ -5,9 +5,9 @@
 #include <unistd.h>
 
 #ifdef STANDALONE_ZAPATA
-#include "zaptel.h"
+#include "kernel/zaptel.h"
 #else
-#include <linux/zaptel.h>
+#include <zaptel/zaptel.h>
 #endif
 
 int main(int argc, char *argv[])
